/******************************************************************************
* Find the column number of a given parameter;
* works both for catalogues with headers and for those with .par files;
* the special parameter name _LAST_COL causes the column number of the last 
* column (= total number of columns) to be returned; 
* this does not work for cats with headers
******************************************************************************/

#include <stdio.h>
#include <string.h>

#define L_WORD 2000
#define N_PARS 200


int colid2colnum(char *parname, char *cat) {

  int            i;
  static int     npars, header;
  char           buffer[L_WORD];
  static char    ocat[L_WORD]={""}, pars[N_PARS][50];
  FILE           *data_file;
  int            strisnlo(char *), ncolumns(FILE *);
  char           *wdinstr(char *, char *, int *);
  FILE           *faskropen(char *, char *, int);
  void           errormsg(char *, ...), nferrormsg(char *, ...);


  if (strcmp(ocat, cat))
    /* Reset header */
    header = 0;

  /* Try to find the parameter in the header first */
  if (strcmp(ocat, cat) || header) {
    /* Open the catalogue */
    if ((data_file = faskropen("Catalogue?", cat, 3)) == NULL)
      errormsg("colid2colnum(): Couldn't open %s!", cat);

    while (fgets(buffer, L_WORD, data_file) != NULL && buffer[0] == '#')
      if (wdinstr(parname, buffer, &i) != NULL) {
	i--;
	header = 1;
	strcpy(ocat, cat);
	fclose(data_file);
	return i;
      }
    fclose(data_file);
    
    if (header) {
      nferrormsg("colid2colnum(): Couldn't find parameter %s!", parname);
      return 0;
    }
  }

  /* Header didn't work, must be a cat with .par file */
  if (strcmp(ocat, cat)) {
    strcpy(ocat, cat);

    /* Read in the parameter names */
    strcpy(buffer, cat);
    strcat(buffer, ".par");

    if ((data_file = faskropen("Parameter file?", buffer, 3)) == NULL)
      errormsg("colid2colnum(): Couldn't open %s!", buffer);
    
    i = 0;
    while (i < N_PARS && fgets(buffer, L_WORD, data_file) != NULL)
      if (buffer[0] != '#')
	if (sscanf(buffer, "%s", pars[i]) == 1 && strisnlo(pars[i]))
	  i++;
    npars = i;
    fclose(data_file);

    /* Check the number of columns in the catalogue */
    if ((data_file = faskropen("Catalogue?", cat, 3)) == NULL)
      errormsg("colid2colnum(): Couldn't open %s!", cat);
    i = ncolumns(data_file);
    fclose(data_file);

    if (i != npars)
      errormsg("colid2colnum(): The numbers of parameters and columns don't\
 match!");
  }

  /* Find the requested parameter */
  if (!strcmp(parname, "_LAST_COL"))
    return npars;

  for (i = 0; i < npars; i++)
    if (!strcmp(parname, pars[i]))
      break;
  if (i == npars) {
    nferrormsg("colid2colnum(): Couldn't find parameter %s!", parname);
    return 0;
  }

  return i+1;
}
